@@ -717,7 +717,7 @@ struct
     else
       mk_exists ctx sorts names body weight patterns nopatterns quantifier_id skolem_id
 
-  let mk_quantifier (ctx:context) (universal:bool) (bound_constants:expr list) (body:expr) (weight:int option) (patterns:Pattern.pattern list) (nopatterns:expr list) (quantifier_id:Symbol.symbol option) (skolem_id:Symbol.symbol option) =
+  let mk_quantifier_const (ctx:context) (universal:bool) (bound_constants:expr list) (body:expr) (weight:int option) (patterns:Pattern.pattern list) (nopatterns:expr list) (quantifier_id:Symbol.symbol option) (skolem_id:Symbol.symbol option) =
     if universal then
       mk_forall_const ctx bound_constants body weight patterns nopatterns quantifier_id skolem_id
     else
@@ -1883,8 +1883,6 @@ struct
     | L_FALSE -> Solver.UNSATISFIABLE
     | _ -> Solver.UNKNOWN
 
-  let push x = Z3native.fixedpoint_push (gc x) x
-  let pop x = Z3native.fixedpoint_pop (gc x) x
   let update_rule x = Z3native.fixedpoint_update_rule (gc x) x
 
   let get_answer x =
