@@ -157,32 +157,25 @@ namespace nlsat {
         ~imp() {
         }
 
-        void display(std::ostream & out, polynomial_ref const & p) const {
+        std::ostream& display(std::ostream & out, polynomial_ref const & p) const {
             m_pm.display(out, p, m_solver.display_proc());
+            return out;
         }
         
-        void display(std::ostream & out, polynomial_ref_vector const & ps, char const * delim = "\n") const {
+        std::ostream& display(std::ostream & out, polynomial_ref_vector const & ps, char const * delim = "\n") const {
             for (unsigned i = 0; i < ps.size(); i++) {
                 if (i > 0)
                     out << delim;
                 m_pm.display(out, ps.get(i), m_solver.display_proc());
             }
+            return out;
         }
         
-        void display(std::ostream & out, literal l) const { m_solver.display(out, l); }
-        void display_var(std::ostream & out, var x) const { m_solver.display(out, x); }
-        void display(std::ostream & out, unsigned sz, literal const * ls) {
-            for (unsigned i = 0; i < sz; i++) {
-                display(out, ls[i]);
-                out << "\n";
-            }
-        }
-        void display(std::ostream & out, literal_vector const & ls) {
-            display(out, ls.size(), ls.c_ptr()); 
-        }
-        void display(std::ostream & out, scoped_literal_vector const & ls) {
-            display(out, ls.size(), ls.c_ptr()); 
-        }
+        std::ostream& display(std::ostream & out, literal l) const { return m_solver.display(out, l); }
+        std::ostream& display_var(std::ostream & out, var x) const { return m_solver.display(out, x); }
+        std::ostream& display(std::ostream & out, unsigned sz, literal const * ls) const { return m_solver.display(out, sz, ls); }
+        std::ostream& display(std::ostream & out, literal_vector const & ls) const { return display(out, ls.size(), ls.c_ptr()); }
+        std::ostream& display(std::ostream & out, scoped_literal_vector const & ls) const { return display(out, ls.size(), ls.c_ptr()); }
 
         /**
            \brief Add literal to the result vector.
@@ -430,7 +423,7 @@ namespace nlsat {
                             bool lit_val  = l.sign() ? !atom_val : atom_val;
                             return lit_val ? true_literal : false_literal;
                         }
-                        else if (s == -1 && a->is_odd(i)) {
+                        else if (s < 0 && a->is_odd(i)) {
                             atom_sign = -atom_sign;
                         }
                         normalized = true;
@@ -1135,7 +1128,7 @@ namespace nlsat {
                                 info.add_lc_ineq();
                             }
                         }
-                        if (s == -1 && !is_even) {
+                        if (s < 0 && !is_even) {
                             atom_sign = -atom_sign;
                         }
                     }
@@ -1162,18 +1155,21 @@ namespace nlsat {
                 new_lit = m_solver.mk_ineq_literal(new_k, new_factors.size(), new_factors.c_ptr(), new_factors_even.c_ptr());
                 if (l.sign())
                     new_lit.neg();
-                TRACE("nlsat_simplify_core", tout << "simplified literal:\n"; display(tout, new_lit); tout << "\n";);
+                TRACE("nlsat_simplify_core", tout << "simplified literal:\n"; display(tout, new_lit) << " " << m_solver.value(new_lit) << "\n";);
+                
                 if (max_var(new_lit) < max) {
-                    // The conflicting core may have redundant literals.
-                    // We should check whether new_lit is true in the current model, and discard it if that is the case
-                    VERIFY(m_solver.value(new_lit) != l_undef);
-                    if (m_solver.value(new_lit) == l_false)
+                    if (m_solver.value(new_lit) == l_true) {
+                        new_lit = l;
+                    }
+                    else {
                         add_literal(new_lit);
-                    new_lit = true_literal;
-                    return;
+                        new_lit = true_literal;
+                    }
+                }
+                else {
+                    new_lit = normalize(new_lit, max);
+                    TRACE("nlsat_simplify_core", tout << "simplified literal after normalization:\n"; display(tout, new_lit); tout << " " << m_solver.value(new_lit) << "\n";);
                 }
-                new_lit = normalize(new_lit, max);
-                TRACE("nlsat_simplify_core", tout << "simplified literal after normalization:\n"; display(tout, new_lit); tout << "\n";);
             }
             else {
                 new_lit = l;
@@ -1333,6 +1329,7 @@ namespace nlsat {
                 poly * eq_p = eq->p(0);
                 VERIFY(simplify(C, eq_p, max));
                 // add equation as an assumption                
+                TRACE("nlsat_simpilfy_core", display(tout << "adding equality as assumption ", literal(eq->bvar(), true)); tout << "\n";);
                 add_literal(literal(eq->bvar(), true));
             }
         }
@@ -1357,9 +1354,9 @@ namespace nlsat {
                 var max = max_var(num, ls);
                 SASSERT(max != null_var);
                 normalize(m_core2, max);
-                TRACE("nlsat_explain", tout << "core after normalization\n"; display(tout, m_core2););
+                TRACE("nlsat_explain", tout << "core after normalization\n"; display(tout, m_core2) << "\n";);
                 simplify(m_core2, max);
-                TRACE("nlsat_explain", tout << "core after simplify\n"; display(tout, m_core2););
+                TRACE("nlsat_explain", tout << "core after simplify\n"; display(tout, m_core2) << "\n";);
                 main(m_core2.size(), m_core2.c_ptr());
                 m_core2.reset();
             }
@@ -1382,14 +1379,14 @@ namespace nlsat {
                 literal l = core[i];
                 atom * a  = m_atoms[l.var()];
                 SASSERT(a != 0);
-                interval_set_ref inf = m_evaluator.infeasible_intervals(a, l.sign());
+                interval_set_ref inf = m_evaluator.infeasible_intervals(a, l.sign(), nullptr);
                 r = ism.mk_union(inf, r);
                 if (ism.is_full(r)) {
                     // Done
                     return false;
                 }
             }
-            TRACE("nlsat_mininize", tout << "interval set after adding partial core:\n" << r << "\n";);
+            TRACE("nlsat_minimize", tout << "interval set after adding partial core:\n" << r << "\n";);
             if (todo.size() == 1) {
                 // Done
                 core.push_back(todo[0]);
@@ -1401,7 +1398,7 @@ namespace nlsat {
                 literal l = todo[i];
                 atom * a  = m_atoms[l.var()];
                 SASSERT(a != 0);
-                interval_set_ref inf = m_evaluator.infeasible_intervals(a, l.sign());
+                interval_set_ref inf = m_evaluator.infeasible_intervals(a, l.sign(), nullptr);
                 r = ism.mk_union(inf, r);
                 if (ism.is_full(r)) {
                     // literal l must be in the core
@@ -1425,15 +1422,15 @@ namespace nlsat {
             todo.reset(); core.reset();
             todo.append(num, ls);
             while (true) {
-                TRACE("nlsat_mininize", tout << "core minimization:\n"; display(tout, todo); tout << "\nCORE:\n"; display(tout, core););
+                TRACE("nlsat_minimize", tout << "core minimization:\n"; display(tout, todo); tout << "\nCORE:\n"; display(tout, core););
                 if (!minimize_core(todo, core))
                     break;
                 std::reverse(todo.begin(), todo.end());
-                TRACE("nlsat_mininize", tout << "core minimization:\n"; display(tout, todo); tout << "\nCORE:\n"; display(tout, core););
+                TRACE("nlsat_minimize", tout << "core minimization:\n"; display(tout, todo); tout << "\nCORE:\n"; display(tout, core););
                 if (!minimize_core(todo, core))
                     break;
             }
-            TRACE("nlsat_mininize", tout << "core:\n"; display(tout, core););
+            TRACE("nlsat_minimize", tout << "core:\n"; display(tout, core););
             r.append(core.size(), core.c_ptr());
         }
 
@@ -1452,12 +1449,12 @@ namespace nlsat {
         void operator()(unsigned num, literal const * ls, scoped_literal_vector & result) {
             SASSERT(check_already_added());
             SASSERT(num > 0);
-            TRACE("nlsat_explain", tout << "[explain] set of literals is infeasible in the current interpretation\n"; display(tout, num, ls););
+            TRACE("nlsat_explain", tout << "[explain] set of literals is infeasible in the current interpretation\n"; display(tout, num, ls) << "\n";);
             m_result = &result;
             process(num, ls);
             reset_already_added();
             m_result = nullptr;
-            TRACE("nlsat_explain", display(tout << "[explain] result\n", result););
+            TRACE("nlsat_explain", display(tout << "[explain] result\n", result) << "\n";);
             CASSERT("nlsat", check_already_added());
         }
 
@@ -1467,10 +1464,7 @@ namespace nlsat {
             m_result = &result;
             svector<literal> lits;
             TRACE("nlsat", tout << "project x" << x << "\n"; 
-                  for (unsigned i = 0; i < num; ++i) {
-                      m_solver.display(tout, ls[i]) << " ";
-                  }
-                  tout << "\n";
+                  m_solver.display(tout, num, ls);
                   m_solver.display(tout););
                   
             DEBUG_CODE(
@@ -1514,12 +1508,7 @@ namespace nlsat {
                 result.set(i, ~result[i]);
             }
             DEBUG_CODE(
-                TRACE("nlsat", 
-                      for (literal l : result) {
-                          m_solver.display(tout << " ", l);
-                      }
-                      tout << "\n";
-                      );
+                TRACE("nlsat", m_solver.display(tout, result.size(), result.c_ptr()) << "\n"; );
                 for (literal l : result) {
                     CTRACE("nlsat", l_true != m_solver.value(l), m_solver.display(tout, l) << " " << m_solver.value(l) << "\n";);
                     SASSERT(l_true == m_solver.value(l));
@@ -1627,12 +1616,12 @@ namespace nlsat {
             unsigned glb_index = 0, lub_index = 0;
             scoped_anum lub(m_am), glb(m_am), x_val(m_am);
             x_val = m_assignment.value(x);
+            bool glb_valid = false, lub_valid = false;
             for (unsigned i = 0; i < ps.size(); ++i) {
                 p = ps.get(i);
                 scoped_anum_vector & roots = m_roots_tmp;
                 roots.reset();
                 m_am.isolate_roots(p, undef_var_assignment(m_assignment, x), roots);
-                bool glb_valid = false, lub_valid = false;
                 for (auto const& r : roots) {
                     int s = m_am.compare(x_val, r);
                     SASSERT(s != 0);
@@ -1640,23 +1629,19 @@ namespace nlsat {
                     if (s < 0 && (!lub_valid || m_am.lt(r, lub))) {
                         lub_index = i;
                         m_am.set(lub, r);
+                        lub_valid = true;
                     }
 
                     if (s > 0 && (!glb_valid || m_am.lt(glb, r))) {
                         glb_index = i;
-                        m_am.set(glb, r);                        
+                        m_am.set(glb, r);
+                        glb_valid = true;
                     }
-                    lub_valid |= s < 0;
-                    glb_valid |= s > 0;
-                }
-                if (glb_valid) {
-                    ++num_glb;
-                }
-                if (lub_valid) {
-                    ++num_lub;
+                    if (s < 0) ++num_lub;
+                    if (s > 0) ++num_glb;
                 }
             }
-            TRACE("nlsat_explain", tout << ps << "\n";);
+            TRACE("nlsat_explain", tout << "glb: " << num_glb << " lub: " << num_lub << "\n" << lub_index << "\n" << glb_index << "\n" << ps << "\n";);
 
             if (num_lub == 0) {
                 project_plus_infinity(x, ps);
