@@ -158,7 +158,7 @@ namespace opt {
         return m_dump_benchmarks;
     }
 
-    lbool opt_solver::check_sat_core(unsigned num_assumptions, expr * const * assumptions) {
+    lbool opt_solver::check_sat_core2(unsigned num_assumptions, expr * const * assumptions) {
         TRACE("opt_verbose", {
             tout << "context size: " << m_context.size() << "\n";            
             for (unsigned i = 0; i < m_context.size(); ++i) {
@@ -208,6 +208,9 @@ namespace opt {
         return m_context.preferred_sat(asms, cores);
     }
 
+    void opt_solver::get_levels(ptr_vector<expr> const& vars, unsigned_vector& depth) {
+        return m_context.get_levels(vars, depth);
+    }
 
 
     /**
@@ -230,7 +233,8 @@ namespace opt {
         get_model(m_model);
         inf_eps val2;
         m_valid_objectives[i] = true;
-        TRACE("opt", tout << (has_shared?"has shared":"non-shared") << " " << val << "\n";);
+        has_shared = true;
+        TRACE("opt", tout << (has_shared?"has shared":"non-shared") << " " << val << " " << blocker << "\n";);
         if (!m_models[i]) {
             set_model(i);
         }
@@ -397,8 +401,7 @@ namespace opt {
             return th.mk_ge(m_fm, v, val);
         }
 
-        if (typeid(smt::theory_rdl) == typeid(opt) &&
-            val.get_infinitesimal().is_zero()) {
+        if (typeid(smt::theory_rdl) == typeid(opt)) {
             smt::theory_rdl& th = dynamic_cast<smt::theory_rdl&>(opt);
             return th.mk_ge(m_fm, v, val);
         }
