@@ -27,7 +27,6 @@ Notes:
 
 --*/
 #include "util/ref_util.h"
-#include "util/cooperate.h"
 #include "ast/ast_smt2_pp.h"
 #include "ast/ast_pp.h"
 #include "ast/pb_decl_plugin.h"
@@ -57,10 +56,10 @@ struct goal2sat::imp {
     svector<sat::literal>       m_result_stack;
     obj_map<app, sat::literal>  m_cache;
     obj_hashtable<expr>         m_interface_vars;
-    sat::solver &               m_solver;
+    sat::solver_core &          m_solver;
     atom2bool_var &             m_map;
     dep2asm_map &               m_dep2asm;
-    sat::bool_var               m_true;
+    sat::literal                m_true;
     bool                        m_ite_extra;
     unsigned long long          m_max_memory;
     expr_ref_vector             m_trail;
@@ -69,7 +68,7 @@ struct goal2sat::imp {
     bool                        m_xor_solver;
     bool                        m_is_lemma;
     
-    imp(ast_manager & _m, params_ref const & p, sat::solver & s, atom2bool_var & map, dep2asm_map& dep2asm, bool default_external):
+    imp(ast_manager & _m, params_ref const & p, sat::solver_core & s, atom2bool_var & map, dep2asm_map& dep2asm, bool default_external):
         m(_m),
         pb(m),
         m_ext(nullptr),
@@ -81,14 +80,14 @@ struct goal2sat::imp {
         m_default_external(default_external),
         m_is_lemma(false) {
         updt_params(p);
-        m_true = sat::null_bool_var;
-        mk_true();
+        m_true = sat::null_literal;
     }
         
     void updt_params(params_ref const & p) {
         m_ite_extra  = p.get_bool("ite_extra", true);
         m_max_memory = megabytes_to_bytes(p.get_uint("max_memory", UINT_MAX));
         m_xor_solver = p.get_bool("xor_solver", false);
+        if (m_xor_solver) ensure_extension();
     }
 
     void throw_op_not_handled(std::string const& s) {
@@ -98,31 +97,31 @@ struct goal2sat::imp {
     
     void mk_clause(sat::literal l) {
         TRACE("goal2sat", tout << "mk_clause: " << l << "\n";);
-        m_solver.mk_clause(1, &l);
+        m_solver.add_clause(1, &l, false);
     }
 
     void set_lemma_mode(bool f) { m_is_lemma = f; }
 
     void mk_clause(sat::literal l1, sat::literal l2) {
         TRACE("goal2sat", tout << "mk_clause: " << l1 << " " << l2 << "\n";);
-        m_solver.mk_clause(l1, l2, m_is_lemma);
+        m_solver.add_clause(l1, l2, m_is_lemma);
     }
 
-    void mk_clause(sat::literal l1, sat::literal l2, sat::literal l3) {
+    void mk_clause(sat::literal l1, sat::literal l2, sat::literal l3, bool is_lemma = false) {
         TRACE("goal2sat", tout << "mk_clause: " << l1 << " " << l2 << " " << l3 << "\n";);
-        m_solver.mk_clause(l1, l2, l3, m_is_lemma);
+        m_solver.add_clause(l1, l2, l3, m_is_lemma || is_lemma);
     }
 
     void mk_clause(unsigned num, sat::literal * lits) {
         TRACE("goal2sat", tout << "mk_clause: "; for (unsigned i = 0; i < num; i++) tout << lits[i] << " "; tout << "\n";);
-        m_solver.mk_clause(num, lits, m_is_lemma);
+        m_solver.add_clause(num, lits, m_is_lemma);
     }
 
-    sat::bool_var mk_true() {
-        if (m_true == sat::null_bool_var) {
+    sat::literal mk_true() {
+        if (m_true == sat::null_literal) {
             // create fake variable to represent true;
-            m_true = m_solver.mk_var(false);
-            mk_clause(sat::literal(m_true, false)); // v is true
+            m_true = sat::literal(m_solver.add_var(false), false);
+            mk_clause(m_true); // v is true
         }
         return m_true;
     }
@@ -133,14 +132,14 @@ struct goal2sat::imp {
         sat::bool_var v = m_map.to_bool_var(t);
         if (v == sat::null_bool_var) {
             if (m.is_true(t)) {
-                l = sat::literal(mk_true(), sign);
+                l = sign ? ~mk_true() : mk_true();
             }
             else if (m.is_false(t)) {
-                l = sat::literal(mk_true(), !sign);
+                l = sign ? mk_true() : ~mk_true();
             }
             else {
                 bool ext = m_default_external || !is_uninterp_const(t) || m_interface_vars.contains(t);
-                sat::bool_var v = m_solver.mk_var(ext);
+                sat::bool_var v = m_solver.add_var(ext);
                 m_map.insert(t, v);
                 l = sat::literal(v, sign);
                 TRACE("sat", tout << "new_var: " << v << ": " << mk_ismt2_pp(t, m) << "\n";);
@@ -248,7 +247,7 @@ struct goal2sat::imp {
         }
         else {
             SASSERT(num <= m_result_stack.size());
-            sat::bool_var k = m_solver.mk_var();
+            sat::bool_var k = m_solver.add_var(false);
             sat::literal  l(k, false);
             m_cache.insert(t, l);
             sat::literal * lits = m_result_stack.end() - num;
@@ -287,7 +286,7 @@ struct goal2sat::imp {
         }
         else {
             SASSERT(num <= m_result_stack.size());
-            sat::bool_var k = m_solver.mk_var();
+            sat::bool_var k = m_solver.add_var(false);
             sat::literal  l(k, false);
             m_cache.insert(t, l);
             // l => /\ lits
@@ -330,7 +329,7 @@ struct goal2sat::imp {
             m_result_stack.reset();
         }
         else {
-            sat::bool_var k = m_solver.mk_var();
+            sat::bool_var k = m_solver.add_var(false);
             sat::literal  l(k, false);
             m_cache.insert(n, l);
             mk_clause(~l, ~c, t);
@@ -338,8 +337,8 @@ struct goal2sat::imp {
             mk_clause(l,  ~c, ~t);
             mk_clause(l,   c, ~e);
             if (m_ite_extra) {
-                mk_clause(~t, ~e, l);
-                mk_clause(t,  e, ~l);
+                mk_clause(~t, ~e, l, false);
+                mk_clause(t,  e, ~l, false);
             }
             m_result_stack.shrink(sz-3);
             if (sign)
@@ -367,7 +366,7 @@ struct goal2sat::imp {
             m_result_stack.reset();
         }
         else {
-            sat::bool_var k = m_solver.mk_var();
+            sat::bool_var k = m_solver.add_var(false);
             sat::literal  l(k, false);
             m_cache.insert(t, l);
             mk_clause(~l, l1, ~l2);
@@ -391,7 +390,7 @@ struct goal2sat::imp {
             return;
         }
         sat::literal_vector lits;
-        sat::bool_var v = m_solver.mk_var(true);
+        sat::bool_var v = m_solver.add_var(true);
         lits.push_back(sat::literal(v, true));
         convert_pb_args(num, lits);
         // ensure that = is converted to xor
@@ -473,7 +472,7 @@ struct goal2sat::imp {
             m_ext->add_pb_ge(sat::null_bool_var, wlits, k1);
         }
         else {
-            sat::bool_var v = m_solver.mk_var(true);
+            sat::bool_var v = m_solver.add_var(true);
             sat::literal lit(v, sign);
             m_ext->add_pb_ge(v, wlits, k.get_unsigned());
             TRACE("goal2sat", tout << "root: " << root << " lit: " << lit << "\n";);
@@ -504,7 +503,7 @@ struct goal2sat::imp {
             m_ext->add_pb_ge(sat::null_bool_var, wlits, k1);
         }
         else {
-            sat::bool_var v = m_solver.mk_var(true);
+            sat::bool_var v = m_solver.add_var(true);
             sat::literal lit(v, sign);
             m_ext->add_pb_ge(v, wlits, k.get_unsigned());
             TRACE("goal2sat", tout << "root: " << root << " lit: " << lit << "\n";);
@@ -519,8 +518,8 @@ struct goal2sat::imp {
         svector<wliteral> wlits;
         convert_pb_args(t, wlits);
         bool base_assert = (root && !sign && m_solver.num_user_scopes() == 0);
-        sat::bool_var v1 = base_assert ? sat::null_bool_var : m_solver.mk_var(true);
-        sat::bool_var v2 = base_assert ? sat::null_bool_var : m_solver.mk_var(true);
+        sat::bool_var v1 = base_assert ? sat::null_bool_var : m_solver.add_var(true);
+        sat::bool_var v2 = base_assert ? sat::null_bool_var : m_solver.add_var(true);
         m_ext->add_pb_ge(v1, wlits, k.get_unsigned());        
         k.neg();
         for (wliteral& wl : wlits) {
@@ -534,7 +533,7 @@ struct goal2sat::imp {
         }
         else {
             sat::literal l1(v1, false), l2(v2, false);
-            sat::bool_var v = m_solver.mk_var();
+            sat::bool_var v = m_solver.add_var(false);
             sat::literal l(v, false);
             mk_clause(~l, l1);
             mk_clause(~l, l2);
@@ -549,12 +548,17 @@ struct goal2sat::imp {
         SASSERT(k.is_unsigned());
         sat::literal_vector lits;
         convert_pb_args(t->get_num_args(), lits);
+        unsigned k2 = k.get_unsigned();
         if (root && m_solver.num_user_scopes() == 0) {
             m_result_stack.reset();
-            m_ext->add_at_least(sat::null_bool_var, lits, k.get_unsigned());
+            if (sign) {
+                for (sat::literal& l : lits) l.neg();
+                k2 = lits.size() + 1 - k2;
+            }
+            m_ext->add_at_least(sat::null_bool_var, lits, k2);
         }
         else {
-            sat::bool_var v = m_solver.mk_var(true);
+            sat::bool_var v = m_solver.add_var(true);
             sat::literal lit(v, false);
             m_ext->add_at_least(v, lits, k.get_unsigned());
             m_cache.insert(t, lit);
@@ -571,14 +575,19 @@ struct goal2sat::imp {
         for (sat::literal& l : lits) {
             l.neg();
         }
+        unsigned k2 = lits.size() - k.get_unsigned();
         if (root && m_solver.num_user_scopes() == 0) {
             m_result_stack.reset();
-            m_ext->add_at_least(sat::null_bool_var, lits, lits.size() - k.get_unsigned());
+            if (sign) {
+                for (sat::literal& l : lits) l.neg();
+                k2 = lits.size() + 1 - k2;
+            }
+            m_ext->add_at_least(sat::null_bool_var, lits, k2);
         }
         else {
-            sat::bool_var v = m_solver.mk_var(true);
+            sat::bool_var v = m_solver.add_var(true);
             sat::literal lit(v, false);
-            m_ext->add_at_least(v, lits, lits.size() - k.get_unsigned());
+            m_ext->add_at_least(v, lits, k2);
             m_cache.insert(t, lit);
             if (sign) lit.neg();
             push_result(root, lit, t->get_num_args());
@@ -589,8 +598,8 @@ struct goal2sat::imp {
         SASSERT(k.is_unsigned());
         sat::literal_vector lits;
         convert_pb_args(t->get_num_args(), lits);
-        sat::bool_var v1 = (root && !sign) ? sat::null_bool_var : m_solver.mk_var(true);
-        sat::bool_var v2 = (root && !sign) ? sat::null_bool_var : m_solver.mk_var(true);
+        sat::bool_var v1 = (root && !sign) ? sat::null_bool_var : m_solver.add_var(true);
+        sat::bool_var v2 = (root && !sign) ? sat::null_bool_var : m_solver.add_var(true);
         m_ext->add_at_least(v1, lits, k.get_unsigned());        
         for (sat::literal& l : lits) {
             l.neg();
@@ -603,7 +612,7 @@ struct goal2sat::imp {
         }
         else {
             sat::literal l1(v1, false), l2(v2, false);
-            sat::bool_var v = m_solver.mk_var();
+            sat::bool_var v = m_solver.add_var(false);
             sat::literal l(v, false);
             mk_clause(~l, l1);
             mk_clause(~l, l2);
@@ -739,7 +748,6 @@ struct goal2sat::imp {
         }
         while (!m_frame_stack.empty()) {
         loop:
-            cooperate("goal2sat");
             if (m.canceled())
                 throw tactic_exception(m.limit().get_cancel_msg());
             if (memory::get_allocation_size() > m_max_memory)
@@ -898,7 +906,7 @@ struct goal2sat::scoped_set_imp {
 };
 
 
-void goal2sat::operator()(goal const & g, params_ref const & p, sat::solver & t, atom2bool_var & m, dep2asm_map& dep2asm, bool default_external, bool is_lemma) {
+void goal2sat::operator()(goal const & g, params_ref const & p, sat::solver_core & t, atom2bool_var & m, dep2asm_map& dep2asm, bool default_external, bool is_lemma) {
     imp proc(g.m(), p, t, m, dep2asm, default_external);
     scoped_set_imp set(this, &proc);
     proc.set_lemma_mode(is_lemma);
@@ -917,7 +925,7 @@ void goal2sat::get_interpreted_atoms(expr_ref_vector& atoms) {
 
 sat2goal::mc::mc(ast_manager& m): m(m), m_var2expr(m) {}
 
-void sat2goal::mc::flush_smc(sat::solver& s, atom2bool_var const& map) {
+void sat2goal::mc::flush_smc(sat::solver_core& s, atom2bool_var const& map) {
     s.flush(m_smc);
     m_var2expr.resize(s.num_vars());
     map.mk_var_inv(m_var2expr);
@@ -926,7 +934,6 @@ void sat2goal::mc::flush_smc(sat::solver& s, atom2bool_var const& map) {
 void sat2goal::mc::flush_gmc() {
     sat::literal_vector updates;
     m_smc.expand(updates);    
-    m_smc.reset();
     if (!m_gmc) m_gmc = alloc(generic_model_converter, m, "sat2goal");
     // now gmc owns the model converter
     sat::literal_vector clause;
@@ -995,44 +1002,11 @@ void sat2goal::mc::get_units(obj_map<expr, bool>& units) {
 }
 
 
+void sat2goal::mc::operator()(sat::model& md) {
+    m_smc(md);
+}
+
 void sat2goal::mc::operator()(model_ref & md) {
-    model_evaluator ev(*md);
-    ev.set_model_completion(false);
-    
-    // create a SAT model using md
-    sat::model sat_md;
-    expr_ref val(m);
-    VERIFY(!m_var2expr.empty());
-    for (expr * atom : m_var2expr) {
-        if (!atom) {
-            sat_md.push_back(l_undef);
-            continue;
-        }
-        ev(atom, val);
-        if (m.is_true(val)) 
-            sat_md.push_back(l_true);
-        else if (m.is_false(val))
-            sat_md.push_back(l_false);
-        else 
-            sat_md.push_back(l_undef);
-    }
-    
-    // apply SAT model converter
-    m_smc(sat_md);
-            
-    // register value of non-auxiliary boolean variables back into md
-    unsigned sz = m_var2expr.size();
-    for (sat::bool_var v = 0; v < sz; v++) {
-        app * atom = m_var2expr.get(v);
-        if (atom && is_uninterp_const(atom)) {
-            func_decl * d = atom->get_decl();
-            lbool new_val = sat_md[v];
-            if (new_val == l_true)
-                md->register_decl(d, m.mk_true());
-            else if (new_val == l_false)
-                md->register_decl(d, m.mk_false());
-        }
-    }    
     // apply externalized model converter
     if (m_gmc) (*m_gmc)(md);
     TRACE("sat_mc", tout << "after sat_mc\n"; model_v2_pp(tout, *md););
@@ -1158,13 +1132,14 @@ struct sat2goal::imp {
         r.assert_expr(fml);
     }
 
-    void assert_clauses(ref<mc>& mc, sat::solver const & s, sat::clause_vector const& clauses, goal & r, bool asserted) {
+    void assert_clauses(ref<mc>& mc, sat::solver_core const & s, sat::clause_vector const& clauses, goal & r, bool asserted) {
         ptr_buffer<expr> lits;
+        unsigned small_lbd = 3; // s.get_config().m_gc_small_lbd;
         for (sat::clause* cp : clauses) {
             checkpoint();
             lits.reset();
             sat::clause const & c = *cp;
-            if (asserted || m_learned || c.glue() <= s.get_config().m_gc_small_lbd) {
+            if (asserted || m_learned || c.glue() <= small_lbd) {
                 for (sat::literal l : c) {
                     lits.push_back(lit2expr(mc, l));
                 }
@@ -1173,11 +1148,11 @@ struct sat2goal::imp {
         }
     }
 
-    sat::ba_solver* get_ba_solver(sat::solver const& s) {
+    sat::ba_solver* get_ba_solver(sat::solver_core const& s) {
         return dynamic_cast<sat::ba_solver*>(s.get_extension());
     }
 
-    void operator()(sat::solver & s, atom2bool_var const & map, goal & r, ref<mc> & mc) {
+    void operator()(sat::solver_core & s, atom2bool_var const & map, goal & r, ref<mc> & mc) {
         if (s.at_base_lvl() && s.inconsistent()) {
             r.assert_expr(m.mk_false());
             return;
@@ -1197,7 +1172,7 @@ struct sat2goal::imp {
 
         // collect binary clauses
         svector<sat::solver::bin_clause> bin_clauses;
-        s.collect_bin_clauses(bin_clauses, m_learned);
+        s.collect_bin_clauses(bin_clauses, m_learned, false);
         for (sat::solver::bin_clause const& bc : bin_clauses) {
             checkpoint();
             r.assert_expr(m.mk_or(lit2expr(mc, bc.first), lit2expr(mc, bc.second)));
@@ -1263,7 +1238,7 @@ struct sat2goal::scoped_set_imp {
     }
 };
 
-void sat2goal::operator()(sat::solver & t, atom2bool_var const & m, params_ref const & p, 
+void sat2goal::operator()(sat::solver_core & t, atom2bool_var const & m, params_ref const & p, 
                           goal & g, ref<mc> & mc) {
     imp proc(g.m(), p);
     scoped_set_imp set(this, &proc);
