@@ -25,13 +25,9 @@ Revision History:
 struct theory_pb_params {
     unsigned m_pb_conflict_frequency;
     bool     m_pb_learn_complements;
-    bool     m_pb_enable_compilation;
-    bool     m_pb_enable_simplex;
     theory_pb_params(params_ref const & p = params_ref()):
         m_pb_conflict_frequency(1000),
-        m_pb_learn_complements(true),
-        m_pb_enable_compilation(true),
-        m_pb_enable_simplex(false)
+        m_pb_learn_complements(true)
     {}
     
     void updt_params(params_ref const & p);
