@@ -27,6 +27,7 @@ Revision History:
 #include "smt/smt_eq_justification.h"
 #include "smt/smt_justification.h"
 #include "smt/smt_bool_var_data.h"
+#include "smt/smt_clause_proof.h"
 #include "smt/smt_theory.h"
 #include "smt/smt_quantifier.h"
 #include "smt/smt_quantifier_stat.h"
@@ -67,6 +68,7 @@ namespace smt {
 
     class context {
         friend class model_generator;
+        friend class lookahead;
     public:
         statistics                  m_stats;
 
@@ -83,6 +85,7 @@ namespace smt {
         setup                       m_setup;
         timer                       m_timer;
         asserted_formulas           m_asserted_formulas;
+        th_rewriter                 m_rewriter;
         scoped_ptr<quantifier_manager>   m_qmanager;
         scoped_ptr<model_generator>      m_model_generator;
         scoped_ptr<relevancy_propagator> m_relevancy_propagator;
@@ -91,6 +94,7 @@ namespace smt {
         mutable unsigned            m_lemma_id;
         progress_callback *         m_progress_callback;
         unsigned                    m_next_progress_sample;
+        clause_proof                m_clause_proof;
 
         region                      m_region;
 
@@ -106,6 +110,8 @@ namespace smt {
 
         unsigned                    m_final_check_idx; // circular counter used for implementing fairness
 
+        bool                        m_is_auxiliary; // used to prevent unwanted information from being logged.
+
         // -----------------------------------
         //
         // Equality & Uninterpreted functions
@@ -256,7 +262,7 @@ namespace smt {
         }
 
         th_rewriter & get_rewriter() {
-            return m_asserted_formulas.get_rewriter();
+            return m_rewriter;
         }
 
         smt_params & get_fparams() {
@@ -299,6 +305,10 @@ namespace smt {
             return m_expr2bool_var[n->get_id()];
         }
 
+        bool_var get_bool_var(enode const * n) const {
+            return get_bool_var(n->get_owner());
+        }
+
         bool_var get_bool_var_of_id(unsigned id) const {
             return m_expr2bool_var[id];
         }
@@ -412,10 +422,19 @@ namespace smt {
             return m_activity[v];
         }
 
-        void set_activity(bool_var v, double & act) {
+        void set_activity(bool_var v, double act) {
             m_activity[v] = act;
         }
 
+        void activity_changed(bool_var v, bool increased) {
+            if (increased) {
+                m_case_split_queue->activity_increased_eh(v);
+            }
+            else {
+                m_case_split_queue->activity_decreased_eh(v);
+            }
+        }
+
         bool is_assumption(bool_var v) const {
             return get_bdata(v).m_assumption;
         }
@@ -509,6 +528,12 @@ namespace smt {
                 result = bool_var2expr(l.var());
         }
 
+        expr_ref literal2expr(literal l) const {
+            expr_ref result(m_manager);
+            literal2expr(l, result);
+            return result;
+        }
+
         bool is_true(enode const * n) const {
             return n == m_true_enode;
         }
@@ -636,7 +661,7 @@ namespace smt {
 
         void remove_cls_occs(clause * cls);
 
-        void del_clause(clause * cls);
+        void del_clause(bool log, clause * cls);
 
         void del_clauses(clause_vector & v, unsigned old_size);
 
@@ -665,9 +690,6 @@ namespace smt {
 
         void remove_watch(bool_var v);
 
-        void mark_as_deleted(clause * cls);
-
-
         // -----------------------------------
         //
         // Internalization
@@ -843,6 +865,9 @@ namespace smt {
 
         void add_lit_occs(clause * cls);
     public:
+
+        void ensure_internalized(expr* e);
+
         void internalize(expr * n, bool gate_ctx);
 
         void internalize(expr * n, bool gate_ctx, unsigned generation);
@@ -1059,8 +1084,6 @@ namespace smt {
 
         enode * get_enode_eq_to(func_decl * f, unsigned num_args, enode * const * args);
 
-        expr* next_decision();
-
     protected:
         bool decide();
 
@@ -1087,7 +1110,7 @@ namespace smt {
             m_bvar_inc *= m_fparams.m_inv_decay;
         }
 
-        bool simplify_clause(clause * cls);
+        bool simplify_clause(clause& cls);
 
         unsigned simplify_clauses(clause_vector & clauses, unsigned starting_at);
 
@@ -1099,7 +1122,7 @@ namespace smt {
         bool is_justifying(clause * cls) const {
             for (unsigned i = 0; i < 2; i++) {
                 b_justification js;
-                js = get_justification(cls->get_literal(i).var());
+                js = get_justification((*cls)[i].var());
                 if (js.get_kind() == b_justification::CLAUSE && js.get_clause() == cls)
                     return true;
             }
@@ -1291,6 +1314,10 @@ namespace smt {
             return display_literals(out, lits.size(), lits.c_ptr());
         }
 
+        std::ostream& display_literal_smt2(std::ostream& out, literal lit) const;
+
+        std::ostream& display_literals_smt2(std::ostream& out, unsigned num_lits, literal const* lits) const;
+
         std::ostream& display_literal_verbose(std::ostream & out, literal lit) const;
 
         std::ostream& display_literals_verbose(std::ostream & out, unsigned num_lits, literal const * lits) const;
@@ -1303,13 +1330,15 @@ namespace smt {
 
         void display_watch_lists(std::ostream & out) const;
 
-        void display_clause_detail(std::ostream & out, clause const * cls) const;
+        std::ostream& display_clause_detail(std::ostream & out, clause const * cls) const;
 
-        void display_clause(std::ostream & out, clause const * cls) const;
+        std::ostream& display_clause(std::ostream & out, clause const * cls) const;
 
-        void display_clauses(std::ostream & out, ptr_vector<clause> const & v) const;
+        std::ostream& display_clause_smt2(std::ostream & out, clause const& cls) const;
 
-        void display_binary_clauses(std::ostream & out) const;
+        std::ostream& display_clauses(std::ostream & out, ptr_vector<clause> const & v) const;
+
+        std::ostream& display_binary_clauses(std::ostream & out) const;
 
         void display_assignment(std::ostream & out) const;
 
@@ -1358,7 +1387,7 @@ namespace smt {
 
         void display_profile(std::ostream & out) const;
 
-        void display(std::ostream& out, b_justification j) const;
+        std::ostream& display(std::ostream& out, b_justification j) const;
 
         // -----------------------------------
         //
@@ -1547,6 +1576,8 @@ namespace smt {
 
         proof * get_proof();
 
+        conflict_resolution& get_cr() { return *m_conflict_resolution.get(); }
+
         void get_relevant_labels(expr* cnstr, buffer<symbol> & result);
 
         void get_relevant_labeled_literals(bool at_lbls, expr_ref_vector & result);
@@ -1575,6 +1606,10 @@ namespace smt {
             return m_unsat_core.get(idx);
         }
 
+        void get_levels(ptr_vector<expr> const& vars, unsigned_vector& depth);
+
+        expr_ref_vector get_trail();
+
         void get_model(model_ref & m) const;
 
         bool update_model(bool refinalize);
