@@ -31,8 +31,6 @@ namespace smt {
             return out << "OK";
         case UNKNOWN:
             return out << "UNKNOWN";
-        case TIMEOUT:
-            return out << "TIMEOUT";
         case MEMOUT:
             return out << "MEMOUT";
         case CANCELED:
@@ -64,7 +62,6 @@ namespace smt {
         std::string r;
         switch(m_last_search_failure) {
         case OK: r = m_unknown; break;
-        case TIMEOUT: r = "timeout"; break;
         case MEMOUT: r = "memout"; break;
         case CANCELED: r = "canceled"; break;
         case NUM_CONFLICTS: r = "max-conflicts-reached"; break;
@@ -104,12 +101,24 @@ namespace smt {
         display_verbose(out, m_manager, num_lits, lits, m_bool_var2expr.c_ptr(), "\n"); return out;
     }
 
-    void context::display_literal_info(std::ostream & out, literal l) const {
-        l.display_compact(out, m_bool_var2expr.c_ptr());
+    std::ostream& context::display_literal_smt2(std::ostream& out, literal l) const {
         if (l.sign())
             out << "  (not " << mk_bounded_pp(bool_var2expr(l.var()), m_manager, 10) << ") ";
         else
             out << "  " << mk_bounded_pp(bool_var2expr(l.var()), m_manager, 10) << " ";
+        return out;
+    }
+
+    std::ostream& context::display_literals_smt2(std::ostream& out, unsigned num_lits, literal const* lits) const {
+        for (unsigned i = 0; i < num_lits; ++i) {
+            display_literal_smt2(out, lits[i]) << "\n";
+        }
+        return out;
+    }
+
+    void context::display_literal_info(std::ostream & out, literal l) const {
+        l.display_compact(out, m_bool_var2expr.c_ptr());
+        display_literal_smt2(out, l);
         out << "relevant: " << is_relevant(bool_var2expr(l.var())) << ", val: " << get_assignment(l) << "\n";
     }
 
@@ -147,30 +156,36 @@ namespace smt {
         }
     }
 
-    void context::display_clause_detail(std::ostream & out, clause const * cls) const {
+    std::ostream& context::display_clause_detail(std::ostream & out, clause const * cls) const {
         out << "lemma: " << cls->is_lemma() << "\n";
-        unsigned num_lits = cls->get_num_literals();
-        for (unsigned i = 0; i < num_lits; i++) {
-            literal l = cls->get_literal(i);
+        for (literal l : *cls) {
             display_literal(out, l);
             out << ", val: " << get_assignment(l) << ", lvl: " << get_assign_level(l)
                 << ", ilvl: " << get_intern_level(l.var()) << ", var: " << l.var() << "\n"
                 << mk_pp(bool_var2expr(l.var()), m_manager) << "\n\n";
         }
+        return out;
     }
 
-    void context::display_clause(std::ostream & out, clause const * cls) const {
+    std::ostream& context::display_clause(std::ostream & out, clause const * cls) const {
         cls->display_compact(out, m_manager, m_bool_var2expr.c_ptr());
+        return out;
     }
 
-    void context::display_clauses(std::ostream & out, ptr_vector<clause> const & v) const {
+    std::ostream& context::display_clause_smt2(std::ostream & out, clause const& cls) const {
+        cls.display_smt2(out, m_manager, m_bool_var2expr.c_ptr());
+        return out;
+    }
+
+    std::ostream& context::display_clauses(std::ostream & out, ptr_vector<clause> const & v) const {
         for (clause* cp : v) {
-            display_clause(out, cp);
+            display_clause_smt2(out, *cp);
             out << "\n";
         }
+        return out;
     }
 
-    void context::display_binary_clauses(std::ostream & out) const {
+    std::ostream& context::display_binary_clauses(std::ostream & out) const {
         bool first = true;
         unsigned l_idx = 0;
         for (watch_list const& wl : m_watches) {
@@ -185,14 +200,22 @@ namespace smt {
                         out << "binary clauses:\n";
                         first = false;
                     }
+                    expr_ref t1(m_manager), t2(m_manager);
+                    literal2expr(neg_l1, t1);
+                    literal2expr(l2, t2);
+                    expr_ref disj(m_manager.mk_or(t1, t2), m_manager);
+                    out << disj << "\n";
+#if 0
                     out << "(clause ";
                     display_literal(out, neg_l1);
                     out << " ";
                     display_literal(out, l2);
                     out << ")\n";
+#endif
                 }
             }
         }
+        return out;
     }
 
     void context::display_assignment(std::ostream & out) const {
@@ -200,6 +223,7 @@ namespace smt {
             out << "current assignment:\n";
             for (literal lit : m_assigned_literals) {
                 display_literal(out, lit);
+                if (!is_relevant(lit)) out << " n ";
                 out << ": ";
                 display_verbose(out, m_manager, 1, &lit, m_bool_var2expr.c_ptr());
                 out << "\n";
@@ -255,7 +279,7 @@ namespace smt {
             for (unsigned i = 0; i < sz; i++) {
                 expr *  n  = m_b_internalized_stack.get(i);
                 bool_var v = get_bool_var_of_id(n->get_id());
-                out << "(#" << n->get_id() << " -> p!" << v << ") ";
+                out << "(#" << n->get_id() << " -> " << literal(v, false) << ") ";
             }
             out << "\n";
         }
@@ -348,9 +372,9 @@ namespace smt {
     }
 
     void context::display_unsat_core(std::ostream & out) const {
-        unsigned sz = m_unsat_core.size();
-        for (unsigned i = 0; i < sz; i++)
-            out << mk_pp(m_unsat_core.get(i), m_manager) << "\n";
+        for (expr* c : m_unsat_core) {
+            out << mk_pp(c, m_manager) << "\n";
+        }
     }
 
     void context::collect_statistics(::statistics & st) const {
@@ -555,41 +579,39 @@ namespace smt {
             }
             out << "\n";
             if (is_app(n)) {
-                for (unsigned i = 0; i < to_app(n)->get_num_args(); i++)
-                    todo.push_back(to_app(n)->get_arg(i));
+                for (expr* arg : *to_app(n)) {
+                    todo.push_back(arg);
+                }
             }
         }
     }
 
-    void context::display(std::ostream& out, b_justification j) const {
+    std::ostream& context::display(std::ostream& out, b_justification j) const {
         switch (j.get_kind()) {
         case b_justification::AXIOM:
             out << "axiom";
             break;
-        case b_justification::BIN_CLAUSE: {
-            literal l2 = j.get_literal();
-            out << "bin-clause ";
-            display_literal(out, l2);
+        case b_justification::BIN_CLAUSE: 
+            out << "bin " << j.get_literal();
             break;
-        }
         case b_justification::CLAUSE: {
             clause * cls = j.get_clause();
             out << "clause ";
             if (cls) out << literal_vector(cls->get_num_literals(), cls->begin());
+            if (cls) display_literals_smt2(out << "\n", cls->get_num_literals(), cls->begin());
             break;
         }
         case b_justification::JUSTIFICATION: {
-            out << "justification " << j.get_justification()->get_from_theory() << ": ";
             literal_vector lits;
             const_cast<conflict_resolution&>(*m_conflict_resolution).justification2literals(j.get_justification(), lits);
-            display_literals(out, lits);
+            out << "justification " << j.get_justification()->get_from_theory() << ": " << lits;
             break;
         }
         default:
             UNREACHABLE();
             break;
         }
-        out << "\n";
+        return out << "\n";
     }
 
     void context::trace_assign(literal l, b_justification j, bool decision) const {
