@@ -196,7 +196,7 @@ namespace smt {
         TRACE("incompleteness_bug", tout << "[internalize-assertion]: #" << n->get_id() << "\n";);
         flet<unsigned> l(m_generation, generation);
         m_stats.m_max_generation = std::max(m_generation, m_stats.m_max_generation);
-        if (get_depth(n) > DEEP_EXPR_THRESHOLD) {
+        if (::get_depth(n) > DEEP_EXPR_THRESHOLD) {
             // if the expression is deep, then execute topological sort to avoid
             // stack overflow.
             // a caveat is that theory internalizers do rely on recursive descent so
@@ -281,7 +281,9 @@ namespace smt {
             set_conflict(mk_justification(justification_proof_wrapper(*this, pr)));
         }
         else {
-            assign(l, mk_justification(justification_proof_wrapper(*this, pr)));
+            justification* j = mk_justification(justification_proof_wrapper(*this, pr));
+            m_clause_proof.add(l, CLS_AUX, j);
+            assign(l, j);
             mark_as_relevant(l);
         }
     }
@@ -317,6 +319,12 @@ namespace smt {
         internalize(n, gate_ctx);
     }
 
+    void context::ensure_internalized(expr* e) {
+        if (!e_internalized(e)) {
+            internalize(e, false);
+        }
+    }
+
     /**
        \brief Internalize the given expression into the logical context.
        
@@ -535,14 +543,10 @@ namespace smt {
     }
 
     void context::internalize_lambda(quantifier * q) {
-        UNREACHABLE();
-
-#if 0
         TRACE("internalize_quantifier", tout << mk_pp(q, m_manager) << "\n";);
         SASSERT(is_lambda(q));
         app_ref lam_name(m_manager.mk_fresh_const("lambda", m_manager.get_sort(q)), m_manager);
-        enode * e = mk_enode(lam_name, true, false, false);
-        expr_ref eq(m_manager), lam_app(m_manager);
+        app_ref eq(m_manager), lam_app(m_manager);
         expr_ref_vector vars(m_manager);
         vars.push_back(lam_name);
         unsigned sz = q->get_num_decls();
@@ -553,10 +557,11 @@ namespace smt {
         lam_app = autil.mk_select(vars.size(), vars.c_ptr());
         eq = m_manager.mk_eq(lam_app, q->get_expr());
         quantifier_ref fa(m_manager);
-        expr * patterns[1] = { m_manager.mk_pattern(lam_name) };
+        expr * patterns[1] = { m_manager.mk_pattern(lam_app) };
         fa = m_manager.mk_forall(sz, q->get_decl_sorts(), q->get_decl_names(), eq, 0, m_manager.lambda_def_qid(), symbol::null, 1, patterns);
         internalize_quantifier(fa, true);
-#endif
+        if (!e_internalized(lam_name)) internalize_uninterpreted(lam_name);
+        m_app2enode.setx(q->get_id(), get_enode(lam_name), nullptr);
     }
 
     /**
@@ -903,6 +908,7 @@ namespace smt {
         expr * n              = m_b_internalized_stack.back();
         unsigned n_id         = n->get_id();
         bool_var v            = get_bool_var_of_id(n_id);
+        m_bool_var2expr[v]    = nullptr;
         TRACE("undo_mk_bool_var", tout << "undo_bool: " << v << "\n" << mk_pp(n, m_manager) << "\n" << "m_bdata.size: " << m_bdata.size()
               << " m_assignment.size: " << m_assignment.size() << "\n";);
         TRACE("mk_var_bug", tout << "undo_mk_bool: " << v << "\n";);
@@ -938,7 +944,7 @@ namespace smt {
             e->mark_as_interpreted();
         TRACE("mk_var_bug", tout << "mk_enode: " << id << "\n";);
         TRACE("generation", tout << "mk_enode: " << id << " " << generation << "\n";);
-        m_app2enode.setx(id, e, 0);
+        m_app2enode.setx(id, e, nullptr);
         m_e_internalized_stack.push_back(n);
         m_trail_stack.push_back(&m_mk_enode_trail);
         m_enodes.push_back(e);
@@ -1019,15 +1025,7 @@ namespace smt {
         sort * s    = term->get_decl()->get_range();
         theory * th = m_theories.get_plugin(s->get_family_id());
         if (th) {
-            if (m_manager.has_trace_stream()) {
-                m_manager.trace_stream() << "[theory-constraints] " << m_manager.get_family_name(s->get_family_id()) << "\n";
-            }
-
             th->apply_sort_cnstr(e, s);
-
-            if (m_manager.has_trace_stream()) {
-                m_manager.trace_stream() << "[end-theory-constraints]\n";
-            }
         }
     }
 
@@ -1313,9 +1311,11 @@ namespace smt {
        The deletion event handler is ignored if binary clause optimization is applicable.
     */
     clause * context::mk_clause(unsigned num_lits, literal * lits, justification * j, clause_kind k, clause_del_eh * del_eh) {
-        TRACE("mk_clause", tout << "creating clause:\n"; display_literals_verbose(tout, num_lits, lits); tout << "\n";);
+        TRACE("mk_clause", display_literals_verbose(tout << "creating clause: " << literal_vector(num_lits, lits) << "\n", num_lits, lits) << "\n";);
+        m_clause_proof.add(num_lits, lits, k, j);
         switch (k) {
-        case CLS_AUX: {
+        case CLS_AUX: 
+        case CLS_TH_AXIOM: {
             literal_buffer simp_lits;
             if (!simplify_aux_clause_literals(num_lits, lits, simp_lits))
                 return nullptr; // clause is equivalent to true;
@@ -1329,7 +1329,7 @@ namespace smt {
             }
             break;
         }
-        case CLS_AUX_LEMMA: {
+        case CLS_TH_LEMMA: {
             if (!simplify_aux_lemma_literals(num_lits, lits))
                 return nullptr; // clause is equivalent to true
             // simplify_aux_lemma_literals does not delete literals assigned to false, so
@@ -1339,11 +1339,11 @@ namespace smt {
         default:
             break;
         }
-        TRACE("mk_clause", tout << "after simplification:\n"; display_literals(tout, num_lits, lits); tout << "\n";);
+        TRACE("mk_clause", tout << "after simplification:\n"; display_literals_verbose(tout, num_lits, lits) << "\n";);
         unsigned activity = 0;
         if (activity == 0)
             activity = 1;
-        bool     lemma = k != CLS_AUX;
+        bool  lemma = is_lemma(k);
         m_stats.m_num_mk_lits += num_lits;
         switch (num_lits) {
         case 0:
@@ -1364,9 +1364,10 @@ namespace smt {
                 literal l2 = lits[1];
                 m_watches[(~l1).index()].insert_literal(l2);
                 m_watches[(~l2).index()].insert_literal(l1);
-                if (get_assignment(l2) == l_false)
+                if (get_assignment(l2) == l_false) {
                     assign(l1, b_justification(~l2));
-
+                }
+                m_clause_proof.add(l1, l2, k, j);
                 m_stats.m_num_mk_bin_clause++;
                 return nullptr;
             }
@@ -1378,6 +1379,7 @@ namespace smt {
             bool reinit         = save_atoms;
             SASSERT(!lemma || j == 0 || !j->in_region());
             clause * cls = clause::mk(m_manager, num_lits, lits, k, j, del_eh, save_atoms, m_bool_var2expr.c_ptr());
+            m_clause_proof.add(*cls);
             if (lemma) {
                 cls->set_activity(activity);
                 if (k == CLS_LEARNED) {
@@ -1385,7 +1387,7 @@ namespace smt {
                     cls->swap_lits(1, w2_idx);
                 }
                 else {
-                    SASSERT(k == CLS_AUX_LEMMA);
+                    SASSERT(k == CLS_TH_LEMMA);
                     int w1_idx = select_watch_lit(cls, 0);
                     cls->swap_lits(0, w1_idx);
                     int w2_idx = select_watch_lit(cls, 1);
@@ -1398,14 +1400,14 @@ namespace smt {
                 add_watch_literal(cls, 1);
                 if (get_assignment(cls->get_literal(0)) == l_false) {
                     set_conflict(b_justification(cls));
-                    if (k == CLS_AUX_LEMMA && m_scope_lvl > m_base_lvl) {
+                    if (k == CLS_TH_LEMMA && m_scope_lvl > m_base_lvl) {
                         reinit     = true;
                         iscope_lvl = m_scope_lvl;
                     }
                 }
                 else if (get_assignment(cls->get_literal(1)) == l_false) {
                     assign(cls->get_literal(0), b_justification(cls));
-                    if (k == CLS_AUX_LEMMA && m_scope_lvl > m_base_lvl) {
+                    if (k == CLS_TH_LEMMA && m_scope_lvl > m_base_lvl) {
                         reinit     = true;
                         iscope_lvl = m_scope_lvl;
                     }
@@ -1434,9 +1436,7 @@ namespace smt {
     }
 
     void context::add_lit_occs(clause * cls) {
-        unsigned num_lits = cls->get_num_literals();
-        for (unsigned i = 0; i < num_lits; i++) {
-            literal l = cls->get_literal(i);
+        for (literal l : *cls) {
             m_lit_occs[l.index()].insert(cls);
         }
     }
@@ -1453,9 +1453,7 @@ namespace smt {
     
     void context::mk_th_axiom(theory_id tid, unsigned num_lits, literal * lits, unsigned num_params, parameter * params) {
         justification * js = nullptr;
-        TRACE("mk_th_axiom", 
-              display_literals_verbose(tout, num_lits, lits);
-              tout << "\n";);
+        TRACE("mk_th_axiom", display_literals_verbose(tout, num_lits, lits) << "\n";);
 
         if (m_manager.proofs_enabled()) {
             js = mk_justification(theory_axiom_justification(tid, m_region, num_lits, lits, num_params, params));
@@ -1466,7 +1464,7 @@ namespace smt {
             SASSERT(tmp.size() == num_lits);
             display_lemma_as_smt_problem(tmp.size(), tmp.c_ptr(), false_literal, m_fparams.m_logic);
         }
-        mk_clause(num_lits, lits, js);
+        mk_clause(num_lits, lits, js, CLS_TH_AXIOM);
     }
     
     void context::mk_th_axiom(theory_id tid, literal l1, literal l2, unsigned num_params, parameter * params) {
