@@ -159,6 +159,12 @@ namespace smt {
             typename vector<row_entry>::const_iterator begin_entries() const { return m_entries.begin(); }
             typename vector<row_entry>::iterator end_entries() { return m_entries.end(); }
             typename vector<row_entry>::const_iterator end_entries() const { return m_entries.end(); }
+
+            typename vector<row_entry>::iterator begin() { return m_entries.begin(); }
+            typename vector<row_entry>::const_iterator begin() const { return m_entries.begin(); }
+            typename vector<row_entry>::iterator end() { return m_entries.end(); }
+            typename vector<row_entry>::const_iterator end() const { return m_entries.end(); }
+
             row_entry & add_row_entry(int & pos_idx);
             void del_row_entry(unsigned idx);
             void compress(vector<column> & cols); 
@@ -198,6 +204,10 @@ namespace smt {
             typename svector<col_entry>::const_iterator begin_entries() const { return m_entries.begin(); }
             typename svector<col_entry>::iterator end_entries() { return m_entries.end(); }
             typename svector<col_entry>::const_iterator end_entries() const { return m_entries.end(); }
+            typename svector<col_entry>::iterator begin() { return m_entries.begin(); }
+            typename svector<col_entry>::const_iterator begin() const { return m_entries.begin(); }
+            typename svector<col_entry>::iterator end() { return m_entries.end(); }
+            typename svector<col_entry>::const_iterator end() const { return m_entries.end(); }
             col_entry & add_col_entry(int & pos_idx);
             void del_col_entry(unsigned idx);
         };
@@ -603,6 +613,7 @@ namespace smt {
         theory_var internalize_to_int(app * n);
         void internalize_is_int(app * n);
         theory_var internalize_numeral(app * n);
+        theory_var internalize_numeral(app * n, numeral const& val);
         theory_var internalize_term_core(app * n);
         void mk_axiom(expr * n1, expr * n2, bool simplify_conseq = true);
         void mk_idiv_mod_axioms(expr * dividend, expr * divisor);
@@ -681,8 +692,6 @@ namespace smt {
         void flush_eh() override;
         void reset_eh() override;
         
-        bool validate_eq_in_model(theory_var v1, theory_var v2, bool is_true) const override;
-
         // -----------------------------------
         //
         // bool_var -> atom mapping
