@@ -87,9 +87,13 @@ namespace smt {
         var_data_full * d_full     = m_var_data_full[v];
         d_full->m_parent_maps.push_back(s);
         m_trail_stack.push(push_back_trail<theory_array, enode *, false>(d_full->m_parent_maps));
-        if (!m_params.m_array_weak && !m_params.m_array_delay_exp_axiom && d->m_prop_upward) {
+        if (!m_params.m_array_delay_exp_axiom && d->m_prop_upward) {
             for (enode * n : d->m_parent_selects) {
                 if (!m_params.m_array_cg || n->is_cgr()) {
+                    if (m_params.m_array_weak) {
+                        found_unsupported_op(s);
+                        break;
+                    }
                     instantiate_select_map_axiom(n, s);
                 }                
             }
@@ -100,14 +104,16 @@ namespace smt {
     // set set_prop_upward on root and recursively on children if necessary.
     // 
     void theory_array_full::set_prop_upward(theory_var v) {
-        if (m_params.m_array_weak)
-            return;
         v = find(v);
         var_data * d = m_var_data[v];
         if (!d->m_prop_upward) {
             m_trail_stack.push(reset_flag_trail<theory_array>(d->m_prop_upward));
             d->m_prop_upward = true;
             TRACE("array", tout << "#" << v << "\n";);
+            if (m_params.m_array_weak) {
+                found_unsupported_op(v);
+                return;
+            }
             if (!m_params.m_array_delay_exp_axiom) {
                 instantiate_axiom2b_for(v);
                 instantiate_axiom_map_for(v);
@@ -134,8 +140,8 @@ namespace smt {
             set_prop_upward(n->get_arg(0)->get_th_var(get_id()));
         }
         else if (is_map(n)) {
-            for (unsigned i = 0; i < n->get_num_args(); ++i) {
-                set_prop_upward(n->get_arg(i)->get_th_var(get_id()));
+            for (enode* arg : enode::args(n)) {
+                set_prop_upward(arg->get_th_var(get_id()));
             }
         }
     }
@@ -250,7 +256,7 @@ namespace smt {
             return theory_array::internalize_term(n);
         }
 
-        if (!is_const(n) && !is_default(n)  && !is_map(n) && !is_as_array(n)) {
+        if (!is_const(n) && !is_default(n)  && !is_map(n) && !is_as_array(n) && !is_set_has_size(n) && !is_set_card(n)) {
             if (!is_array_ext(n))
                 found_unsupported_op(n);
             return false;
@@ -274,6 +280,12 @@ namespace smt {
                 mk_var(arg0);
             }
         }
+        else if (is_set_has_size(n) || is_set_card(n)) {
+            if (!m_bapa) {
+                m_bapa = alloc(theory_array_bapa, *this);
+            }
+            m_bapa->internalize_term(n);
+        }
 
         enode* node = ctx.get_enode(n);
         if (!is_attached_to_var(node)) {
@@ -343,14 +355,20 @@ namespace smt {
         SASSERT(v != null_theory_var);
         v = find(v);
         var_data* d = m_var_data[v];
-        TRACE("array", tout << "v" << v << " " << d->m_prop_upward << " " << m_params.m_array_delay_exp_axiom << "\n";);
+        TRACE("array", tout << "v" << v << " " << mk_pp(get_enode(v)->get_owner(), get_manager()) << " " 
+              << d->m_prop_upward << " " << m_params.m_array_delay_exp_axiom << "\n";);
         for (enode * store : d->m_stores) {
             SASSERT(is_store(store));
             instantiate_default_store_axiom(store);
         }        
 
-        if (!m_params.m_array_weak && !m_params.m_array_delay_exp_axiom && d->m_prop_upward) {
-            instantiate_parent_stores_default(v);
+        if (!m_params.m_array_delay_exp_axiom && d->m_prop_upward) {
+            if (m_params.m_array_weak) {
+                found_unsupported_op(v);
+            }
+            else {
+                instantiate_parent_stores_default(v);
+            }
         }
     }
 
@@ -370,10 +388,14 @@ namespace smt {
             SASSERT(is_map(map));
             instantiate_select_map_axiom(s, map);
         }
-        if (!m_params.m_array_weak && !m_params.m_array_delay_exp_axiom && d->m_prop_upward) {
+        if (!m_params.m_array_delay_exp_axiom && d->m_prop_upward) {
             for (enode * map : d_full->m_parent_maps) {
                 SASSERT(is_map(map));
                 if (!m_params.m_array_cg || map->is_cgr()) {
+                    if (m_params.m_array_weak) {
+                        found_unsupported_op(s);
+                        break;
+                    }
                     instantiate_select_map_axiom(s, map);
                 }
             }
@@ -421,6 +443,13 @@ namespace smt {
         }
     }
 
+    bool theory_array_full::should_research(expr_ref_vector & unsat_core) {
+        return m_bapa && m_bapa->should_research(unsat_core);
+    }
+
+    void theory_array_full::add_theory_assumptions(expr_ref_vector & assumptions) {
+        if (m_bapa) m_bapa->add_theory_assumptions(assumptions);
+    }
        
     //
     // Assert axiom: 
@@ -453,12 +482,12 @@ namespace smt {
               tout << mk_bounded_pp(sl->get_owner(), get_manager()) << "\n";);
         unsigned num_args   = select->get_num_args();
         unsigned num_arrays = map->get_num_args();
-        ptr_buffer<expr>       args1, args2;
+        ptr_buffer<expr> args1, args2;
         vector<ptr_vector<expr> > args2l;
         args1.push_back(map);
-        for (unsigned j = 0; j < num_arrays; ++j) {
+        for (expr* ar : *map) {
             ptr_vector<expr> arg;
-            arg.push_back(map->get_arg(j));
+            arg.push_back(ar);
             args2l.push_back(arg);
         }
         for (unsigned i = 1; i < num_args; ++i) {
@@ -509,8 +538,8 @@ namespace smt {
         func_decl* f = to_func_decl(map->get_decl()->get_parameter(0).get_ast());
         SASSERT(map->get_num_args() == f->get_arity());
         ptr_buffer<expr> args2;
-        for (unsigned i = 0; i < map->get_num_args(); ++i) {
-            args2.push_back(mk_default(map->get_arg(i)));
+        for (expr* arg : *map) {
+            args2.push_back(mk_default(arg));
         }
 
         expr_ref def2(m.mk_app(f, args2.size(), args2.c_ptr()), m);
@@ -662,7 +691,7 @@ namespace smt {
         app* store_app = store->get_owner();
         context& ctx = get_context();
         ast_manager& m = get_manager();
-        if (!ctx.add_fingerprint(this, m_default_store_fingerprint, 1, &store)) {
+        if (!ctx.add_fingerprint(this, m_default_store_fingerprint, store->get_num_args(), store->get_args())) {
             return false;
         }
 
@@ -695,15 +724,6 @@ namespace smt {
             eq = mk_and(eqs);
             expr* defA = mk_default(store_app->get_arg(0));
             def2 = m.mk_ite(eq, store_app->get_arg(num_args-1), defA); 
-#if 0
-            // 
-            // add soft constraints to guide model construction so that 
-            // epsilon agrees with the else case in the model construction.
-            // 
-            for (unsigned i = 0; i < eqs.size(); ++i) {
-                // assume_diseq(eqs[i]);
-            }
-#endif 
         }
 
         def1 = mk_default(store_app);
@@ -735,8 +755,11 @@ namespace smt {
                 var_data * d = m_var_data[v];
                 if (d->m_prop_upward && instantiate_axiom_map_for(v))
                     r = FC_CONTINUE;
-                if (d->m_prop_upward && !m_params.m_array_weak) {
-                    if (instantiate_parent_stores_default(v))
+                if (d->m_prop_upward) {
+                    if (m_params.m_array_weak) {
+                        found_unsupported_op(v);
+                    }
+                    else if (instantiate_parent_stores_default(v))
                         r = FC_CONTINUE;
                 }
             }
@@ -748,6 +771,10 @@ namespace smt {
             assert_axiom(eq);
             r = FC_CONTINUE;
         }
+        if (r == FC_DONE && m_bapa) {
+            r = m_bapa->final_check();
+        }
+
         if (r == FC_DONE && m_found_unsupported_op)
             r = FC_GIVEUP;
         return r;
@@ -760,11 +787,10 @@ namespace smt {
         var_data* d = m_var_data[v];
         bool result = false;
         for (enode * store : d->m_parent_stores) {
-            TRACE("array", tout << expr_ref(store->get_owner(), get_manager()) << "\n";);
             SASSERT(is_store(store));
-            if (!m_params.m_array_cg || store->is_cgr()) {
-                if (instantiate_default_store_axiom(store))
-                    result = true;
+            if ((!m_params.m_array_cg || store->is_cgr()) && 
+                instantiate_default_store_axiom(store)) {
+                result = true;
             }
         }
         return result;
@@ -781,8 +807,10 @@ namespace smt {
         else {
             m_eqs.insert(v1, v2, true);
             literal eq(mk_eq(v1, v2, true));
+            if (get_manager().has_trace_stream()) log_axiom_instantiation(get_context().bool_var2expr(eq.var()));
             get_context().mark_as_relevant(eq);            
             assert_axiom(eq);
+            if (get_manager().has_trace_stream()) get_manager().trace_stream() << "[end-of-instance]\n";
 
             // m_eqsv.push_back(eq);
             return true;
