@@ -19,10 +19,11 @@ Revision History:
 #ifndef THEORY_DATATYPE_H_
 #define THEORY_DATATYPE_H_
 
-#include "smt/smt_theory.h"
 #include "util/union_find.h"
-#include "smt/params/theory_datatype_params.h"
+#include "ast/array_decl_plugin.h"
 #include "ast/datatype_decl_plugin.h"
+#include "smt/smt_theory.h"
+#include "smt/params/theory_datatype_params.h"
 #include "smt/proto_model/datatype_factory.h"
 
 namespace smt {
@@ -48,6 +49,7 @@ namespace smt {
         
         theory_datatype_params &  m_params;
         datatype_util             m_util;
+        array_util                m_autil;
         ptr_vector<var_data>      m_var_data;
         th_union_find             m_find;
         th_trail_stack            m_trail_stack;
@@ -73,7 +75,7 @@ namespace smt {
         void sign_recognizer_conflict(enode * c, enode * r);
 
         typedef enum { ENTER, EXIT } stack_op;
-        typedef map<enode*, enode*, obj_ptr_hash<enode>, ptr_eq<enode> > parent_tbl;
+        typedef obj_map<enode, enode*> parent_tbl;
         typedef std::pair<stack_op, enode*> stack_entry;
 
         ptr_vector<enode>     m_to_unmark;
@@ -89,6 +91,8 @@ namespace smt {
         bool oc_cycle_free(enode * n) const { return n->get_root()->is_marked2(); }
 
         void oc_push_stack(enode * n);
+        ptr_vector<enode> m_array_args;
+        ptr_vector<enode> const& get_array_args(enode* n);
 
         // class for managing state of final_check
         class final_check_st {
@@ -102,6 +106,7 @@ namespace smt {
         bool occurs_check(enode * n);
         bool occurs_check_enter(enode * n);
         void occurs_check_explain(enode * top, enode * root);
+        void explain_is_child(enode* parent, enode* child);
 
         void mk_split(theory_var v);
 
