@@ -598,7 +598,9 @@ void theory_diff_logic<Ext>::new_edge(dl_var src, dl_var dst, unsigned num_edges
         le = m_util.mk_le(m_util.mk_add(n2,n1), n3);
         le = get_manager().mk_not(le);
     }
+    if (get_manager().has_trace_stream())log_axiom_instantiation(le);
     ctx.internalize(le, false);
+    if (get_manager().has_trace_stream()) get_manager().trace_stream() << "[end-of-instance]\n";
     ctx.mark_as_relevant(le.get());
     literal lit(ctx.get_literal(le));
     bool_var bv = lit.var();
@@ -628,7 +630,7 @@ void theory_diff_logic<Ext>::new_edge(dl_var src, dl_var dst, unsigned num_edges
                    lits.size(), lits.c_ptr(), 
                    params.size(), params.c_ptr());
     }
-    ctx.mk_clause(lits.size(), lits.c_ptr(), js, CLS_AUX_LEMMA, nullptr);
+    ctx.mk_clause(lits.size(), lits.c_ptr(), js, CLS_TH_LEMMA, nullptr);
     if (dump_lemmas()) {
         symbol logic(m_is_lia ? "QF_LIA" : "QF_LRA");
         ctx.display_lemma_as_smt_problem(lits.size(), lits.c_ptr(), false_literal, logic);
@@ -909,12 +911,6 @@ model_value_proc * theory_diff_logic<Ext>::mk_value(enode * n, model_generator &
     return alloc(expr_wrapper_proc, m_factory->mk_num_value(num, m_util.is_int(n->get_owner())));
 }
 
-template<typename Ext>
-bool theory_diff_logic<Ext>::validate_eq_in_model(theory_var v1, theory_var v2, bool is_true) const {
-    NOT_IMPLEMENTED_YET();
-    return true;
-}
-
 
 template<typename Ext>
 void theory_diff_logic<Ext>::display(std::ostream & out) const {
@@ -1007,6 +1003,11 @@ void theory_diff_logic<Ext>::new_eq_or_diseq(bool is_eq, theory_var v1, theory_v
         t2 = m_util.mk_numeral(k, m.get_sort(s2.get()));
         // t1 - s1 = k
         eq = m.mk_eq(s2.get(), t2.get());
+        if (m.has_trace_stream()) {
+            app_ref body(m);
+            body = m.mk_eq(m.mk_eq(m_util.mk_add(s1, t2), t1), eq);
+            log_axiom_instantiation(body);
+        }
         
         TRACE("diff_logic", 
               tout << v1 << " .. " << v2 << "\n";
@@ -1015,6 +1016,8 @@ void theory_diff_logic<Ext>::new_eq_or_diseq(bool is_eq, theory_var v1, theory_v
         if (!internalize_atom(eq.get(), false)) {
             UNREACHABLE();
         }
+
+        if (m.has_trace_stream()) get_manager().trace_stream() << "[end-of-instance]\n";
                 
         literal l(ctx.get_literal(eq.get()));
         if (!is_eq) {
@@ -1109,8 +1112,8 @@ unsigned theory_diff_logic<Ext>::simplex2edge(unsigned e) {
 
 template<typename Ext> 
 void theory_diff_logic<Ext>::update_simplex(Simplex& S) {
-    unsynch_mpq_manager mgr;
     unsynch_mpq_inf_manager inf_mgr;
+    unsynch_mpq_manager& mgr = inf_mgr.get_mpq_manager();
     unsigned num_nodes = m_graph.get_num_nodes();
     vector<dl_edge<GExt> > const& es = m_graph.get_all_edges();
     S.ensure_var(num_simplex_vars());
@@ -1118,7 +1121,8 @@ void theory_diff_logic<Ext>::update_simplex(Simplex& S) {
         numeral const& a = m_graph.get_assignment(i);
         rational fin = a.get_rational().to_rational();
         rational inf = a.get_infinitesimal().to_rational();
-        mpq_inf q(mgr.dup(fin.to_mpq()), mgr.dup(inf.to_mpq()));
+        mpq_inf q;
+        inf_mgr.set(q, fin.to_mpq(), inf.to_mpq());
         S.set_value(node2simplex(i), q);
         inf_mgr.del(q);
     }
@@ -1149,7 +1153,8 @@ void theory_diff_logic<Ext>::update_simplex(Simplex& S) {
             numeral const& w = e.get_weight();
             rational fin = w.get_rational().to_rational();
             rational inf = w.get_infinitesimal().to_rational();
-            mpq_inf q(mgr.dup(fin.to_mpq()), mgr.dup(inf.to_mpq()));
+            mpq_inf q;
+            inf_mgr.set(q, fin.to_mpq(), inf.to_mpq());
             S.set_upper(base_var, q);
             inf_mgr.del(q);
         }
@@ -1305,7 +1310,6 @@ expr_ref theory_diff_logic<Ext>::mk_ineq(theory_var v, inf_eps const& val, bool
         if (is_strict) {
             f = m.mk_not(f);
         }
-        TRACE("arith", tout << "block: " << f << "\n";);
         return f;
     }
 
