@@ -76,7 +76,7 @@ namespace smt {
             
             void get_dependencies(buffer<smt::model_value_dependency> & result) override {}
             
-            app * mk_value(smt::model_generator & mg, ptr_vector<expr> & ) override {
+            app * mk_value(smt::model_generator & mg, expr_ref_vector const & ) override {
                 smt::context& ctx = m_th.get_context();
                 app* result = nullptr;
                 expr* n = m_node->get_owner();
@@ -256,6 +256,11 @@ namespace smt {
             lt = u().mk_lt(x,y);
             le = b().mk_ule(m().mk_app(r,y),m().mk_app(r,x)); 
             context& ctx = get_context();
+            if (m().has_trace_stream()) {
+                app_ref body(m());
+                body = m().mk_eq(lt, le);
+                log_axiom_instantiation(body);
+            }
             ctx.internalize(lt, false);
             ctx.internalize(le, false);
             literal lit1(ctx.get_literal(lt));
@@ -266,12 +271,15 @@ namespace smt {
             literal lits2[2] = { ~lit1, ~lit2 };
             ctx.mk_th_axiom(get_id(), 2, lits1);
             ctx.mk_th_axiom(get_id(), 2, lits2);
+            if (m().has_trace_stream()) m().trace_stream() << "[end-of-instance]\n";
         }
 
         void assert_cnstr(expr* e) {
             TRACE("theory_dl", tout << mk_pp(e, m()) << "\n";);
             context& ctx = get_context();
+            if (m().has_trace_stream()) log_axiom_instantiation(e);
             ctx.internalize(e, false);
+            if (m().has_trace_stream()) m().trace_stream() << "[end-of-instance]\n";
             literal lit(ctx.get_literal(e));
             ctx.mark_as_relevant(lit);
             ctx.mk_th_axiom(get_id(), 1, &lit);
