@@ -21,10 +21,12 @@ Revision History:
 #include<sstream>
 
 #ifdef _WINDOWS
+#if defined(_MSC_VER)
 #pragma float_control( except, on )   // exception semantics; this does _not_ mean that exceptions are enabled (we want them off!)
 #pragma float_control( precise, on )  // precise semantics (no guessing!)
 #pragma fp_contract(off)              // contractions off (`contraction' means x*y+z is turned into a fused-mul-add).
 #pragma fenv_access(on)               // fpu environment sensitivity (needed to be allowed to make FPU mode changes).
+#endif
 #else
 #include<fenv.h>
 #endif
@@ -253,42 +255,11 @@ void hwf_manager::div(mpf_rounding_mode rm, hwf const & x, hwf const & y, hwf &
 #endif
 }
 
-#ifdef _M_IA64
-#pragma fp_contract(on)
-#endif
-
 void hwf_manager::fma(mpf_rounding_mode rm, hwf const & x, hwf const & y, hwf const &z, hwf & o) {
-    // CMW: fused_mul_add is not available on most CPUs. As of 2012, only Itanium,
-    // Intel Sandybridge and AMD Bulldozers support that (via AVX).
-
     set_rounding_mode(rm);
-
-#ifdef _M_IA64
-    // IA64 (Itanium) will do it, if contractions are on.
-    o.value = x.value * y.value + z.value;
-#else
-#if defined(_WINDOWS)
-#if _MSC_VER >= 1800
     o.value = ::fma(x.value, y.value, z.value);
-#else // Windows, older than VS 2013
-  #ifdef USE_INTRINSICS
-      _mm_store_sd(&o.value, _mm_fmadd_sd(_mm_set_sd(x.value), _mm_set_sd(y.value), _mm_set_sd(z.value)));
-  #else
-      // If all else fails, we are imprecise.
-      o.value = (x.value * y.value) + z;
-  #endif
-#endif
-#else
-    // Linux, macOS
-    o.value = ::fma(x.value, y.value, z.value);
-#endif
-#endif
 }
 
-#ifdef _M_IA64
-#pragma fp_contract(off)
-#endif
-
 void hwf_manager::sqrt(mpf_rounding_mode rm, hwf const & x, hwf & o) {
     set_rounding_mode(rm);
 #ifdef USE_INTRINSICS
@@ -306,7 +277,10 @@ void hwf_manager::round_to_integral(mpf_rounding_mode rm, hwf const & x, hwf & o
     // According to the Intel Architecture manual, the x87-instruction FRNDINT is the
     // same in 32-bit and 64-bit mode. The _mm_round_* intrinsics are SSE4 extensions.
 #ifdef _WINDOWS
-#if defined(USE_INTRINSICS) && \
+#if defined( __MINGW32__ ) && ( defined( __GNUG__ ) || defined( __clang__ ) )
+    o.value = nearbyint(x.value);
+#else
+  #if defined(USE_INTRINSICS) &&                                           \
     (defined(_WINDOWS) && (defined(__AVX__) || defined(_M_X64))) || \
     (!defined(_WINDOWS) && defined(__SSE4_1__))
     switch (rm) {
@@ -320,7 +294,7 @@ void hwf_manager::round_to_integral(mpf_rounding_mode rm, hwf const & x, hwf & o
     default:
         UNREACHABLE(); // Unknown rounding mode.
     }
-#else
+  #else
     double xv = x.value;
     double & ov = o.value;
 
@@ -329,6 +303,7 @@ void hwf_manager::round_to_integral(mpf_rounding_mode rm, hwf const & x, hwf & o
         frndint
         fstp    ov // Store result away.
     }
+  #endif
 #endif
 #else
     // Linux, macOS.
