@@ -178,6 +178,10 @@ public:
 
     constraint_index add_var_bound(var_index j, lconstraint_kind kind, const mpq & right_side) ;
 
+    bool compare_values(var_index j, lconstraint_kind kind, const mpq & right_side);
+
+    bool compare_values(impq const& lhs, lconstraint_kind k, const mpq & rhs);
+
     void update_column_type_and_bound(var_index j, lconstraint_kind kind, const mpq & right_side, constraint_index constr_index);
 
     void add_var_bound_on_constraint_for_term(var_index j, lconstraint_kind kind, const mpq & right_side, constraint_index ci);
@@ -310,7 +314,7 @@ public:
                                              impq &term_max);    
     // starting from a given feasible state look for the maximum of the term
     // return true if found and false if unbounded
-    lp_status maximize_term(unsigned ext_j ,
+    lp_status maximize_term(unsigned j_or_term,
                             impq &term_max);
     
 
@@ -359,7 +363,14 @@ public:
     void detect_rows_with_changed_bounds_for_column(unsigned j);
     
     void detect_rows_with_changed_bounds();
+    inline bool is_base(unsigned j) const {
+        return m_mpq_lar_core_solver.m_r_heading[j] >= 0;
+    }
 
+    bool move_non_basic_columns_to_bounds();
+
+    bool move_non_basic_column_to_bounds(unsigned j);
+    void set_value_for_nbasic_column(unsigned j, const impq & new_val);
     void update_x_and_inf_costs_for_columns_with_changed_bounds();
 
     void update_x_and_inf_costs_for_columns_with_changed_bounds_tableau();
@@ -457,6 +468,7 @@ public:
 
     std::ostream& print_implied_bound(const implied_bound& be, std::ostream & out) const;
 
+    std::ostream& print_values(std::ostream& out) const;
     
     mpq get_left_side_val(const lar_base_constraint &  cns, const std::unordered_map<var_index, mpq> & var_map) const;
 
