@@ -19,8 +19,8 @@ Revision History:
 #ifndef MPQ_H_
 #define MPQ_H_
 
-#include"mpz.h"
-#include"trace.h"
+#include "util/mpz.h"
+#include "util/trace.h"
 
 class mpq {
     mpz m_num;
@@ -31,6 +31,7 @@ class mpq {
 public:
     mpq(int v):m_num(v), m_den(1) {}
     mpq():m_den(1) {}
+    mpq(mpq && other) : m_num(std::move(other.m_num)), m_den(std::move(other.m_den)) {}
     void swap(mpq & other) { m_num.swap(other.m_num); m_den.swap(other.m_den); }
     mpz const & numerator() const { return m_num; }
     mpz const & denominator() const { return m_den; }
@@ -40,12 +41,12 @@ inline void swap(mpq & m1, mpq & m2) { m1.swap(m2); }
 
 template<bool SYNCH = true>
 class mpq_manager : public mpz_manager<SYNCH> {
-    mpz m_n_tmp;
-    mpz m_add_tmp1;
-    mpz m_add_tmp2;
-    mpq m_addmul_tmp;
-    mpq m_lt_tmp1;
-    mpq m_lt_tmp2;
+    mpz m_tmp1;
+    mpz m_tmp2;
+    mpz m_tmp3;
+    mpz m_tmp4;
+    mpq m_q_tmp1;
+    mpq m_q_tmp2;
 
     void reset_denominator(mpq & a) {
         del(a.m_den);
@@ -65,35 +66,15 @@ class mpq_manager : public mpz_manager<SYNCH> {
             del(tmp);
         }
         else {
-            gcd(a.m_num, a.m_den, m_n_tmp);
-            if (is_one(m_n_tmp))
+            gcd(a.m_num, a.m_den, m_tmp1);
+            if (is_one(m_tmp1))
                 return;
-            div(a.m_num, m_n_tmp, a.m_num);
-            div(a.m_den, m_n_tmp, a.m_den);
+            div(a.m_num, m_tmp1, a.m_num);
+            div(a.m_den, m_tmp1, a.m_den);
         }
     }
 
-    void rat_add(mpq const & a, mpq const & b, mpq & c) {
-        STRACE("rat_mpq", tout << "[mpq] " << to_string(a) << " + " << to_string(b) << " == ";); 
-        if (SYNCH) {
-            mpz tmp1, tmp2;
-            mul(a.m_num, b.m_den, tmp1);
-            mul(b.m_num, a.m_den, tmp2);
-            mul(a.m_den, b.m_den, c.m_den);
-            add(tmp1, tmp2, c.m_num);
-            normalize(c);
-            del(tmp1);
-            del(tmp2);
-        }
-        else {
-            mul(a.m_num, b.m_den, m_add_tmp1);
-            mul(b.m_num, a.m_den, m_add_tmp2);
-            mul(a.m_den, b.m_den, c.m_den);
-            add(m_add_tmp1, m_add_tmp2, c.m_num);
-            normalize(c);
-        }
-        STRACE("rat_mpq", tout << to_string(c) << "\n";);
-    }
+    void rat_add(mpq const & a, mpq const & b, mpq & c);
 
     void rat_add(mpq const & a, mpz const & b, mpq & c) {
         STRACE("rat_mpq", tout << "[mpq] " << to_string(a) << " + " << to_string(b) << " == ";); 
@@ -106,54 +87,27 @@ class mpq_manager : public mpz_manager<SYNCH> {
             del(tmp1);
         }
         else {
-            mul(b, a.m_den, m_add_tmp1);
+            mul(b, a.m_den, m_tmp1);
             set(c.m_den, a.m_den);
-            add(a.m_num, m_add_tmp1, c.m_num);
+            add(a.m_num, m_tmp1, c.m_num);
             normalize(c);
         }
         STRACE("rat_mpq", tout << to_string(c) << "\n";);
     }
 
-    void rat_sub(mpq const & a, mpq const & b, mpq & c) {
-        STRACE("rat_mpq", tout << "[mpq] " << to_string(a) << " - " << to_string(b) << " == ";); 
-        if (SYNCH) {
-            mpz tmp1, tmp2;
-            mul(a.m_num, b.m_den, tmp1);
-            mul(b.m_num, a.m_den, tmp2);
-            mul(a.m_den, b.m_den, c.m_den);
-            sub(tmp1, tmp2, c.m_num);
-            normalize(c);
-            del(tmp1);
-            del(tmp2);
-        }
-        else {
-            mul(a.m_num, b.m_den, m_add_tmp1);
-            mul(b.m_num, a.m_den, m_add_tmp2);
-            mul(a.m_den, b.m_den, c.m_den);
-            sub(m_add_tmp1, m_add_tmp2, c.m_num);
-            normalize(c);
-        }
-        STRACE("rat_mpq", tout << to_string(c) << "\n";);
-    }
+    void rat_sub(mpq const & a, mpq const & b, mpq & c);
 
-    void rat_mul(mpq const & a, mpq const & b, mpq & c) {
-        STRACE("rat_mpq", tout << "[mpq] " << to_string(a) << " * " << to_string(b) << " == ";); 
-        mul(a.m_num, b.m_num, c.m_num);
-        mul(a.m_den, b.m_den, c.m_den);
-        normalize(c);
-        STRACE("rat_mpq", tout << to_string(c) << "\n";);
-    }
+    void rat_mul(mpq const & a, mpq const & b, mpq & c);
 
-    void rat_mul(mpz const & a, mpq const & b, mpq & c) {
-        STRACE("rat_mpq", tout << "[mpq] " << to_string(a) << " * " << to_string(b) << " == ";); 
-        mul(a, b.m_num, c.m_num);
-        set(c.m_den, b.m_den);
-        normalize(c);
-        STRACE("rat_mpq", tout << to_string(c) << "\n";);
-    }
+    void rat_mul(mpz const & a, mpq const & b, mpq & c);
 
     bool rat_lt(mpq const & a, mpq const & b);
 
+    template<bool SUB>
+    void lin_arith_op(mpq const& a, mpq const& b, mpq& c, mpz& g, mpz& tmp1, mpz& tmp2, mpz& tmp3);
+
+    void rat_mul(mpq const & a, mpq const & b, mpq & c, mpz& g1, mpz& g2, mpz& tmp1, mpz& tmp2);
+
 public:
     typedef mpq numeral;
     typedef mpq rational;
@@ -174,7 +128,7 @@ public:
 
     static bool is_small(mpz const & a) { return mpz_manager<SYNCH>::is_small(a); }
 
-    static bool is_small(mpq const & a) { return is_small(a.m_num) && is_small(a.m_den); }
+    static bool is_small(mpq const & a) { return is_small(a.m_num) && is_small(a.m_den); }    
 
     static mpq mk_q(int v) { return mpq(v); }
 
@@ -265,7 +219,7 @@ public:
 
     void display_smt2(std::ostream & out, mpq const & a, bool decimal) const;
 
-    void display_decimal(std::ostream & out, mpq const & a, unsigned prec);
+    void display_decimal(std::ostream & out, mpq const & a, unsigned prec, bool truncate = false);
 
     void add(mpz const & a, mpz const & b, mpz & c) { mpz_manager<SYNCH>::add(a, b, c); }
     
@@ -366,8 +320,8 @@ public:
                 del(tmp);
             }
             else {
-                mul(b,c,m_addmul_tmp);
-                add(a, m_addmul_tmp, d);
+                mul(b, c, m_q_tmp1);
+                add(a, m_q_tmp1, d);
             }
         }
     }
@@ -388,8 +342,8 @@ public:
                 del(tmp);
             }
             else {
-                mul(b,c,m_addmul_tmp);
-                add(a, m_addmul_tmp, d);
+                mul(b,c, m_q_tmp1);
+                add(a, m_q_tmp1, d);
             }
         }
     }
@@ -411,8 +365,8 @@ public:
                 del(tmp);
             }
             else {
-                mul(b,c,m_addmul_tmp);
-                sub(a, m_addmul_tmp, d);
+                mul(b,c, m_q_tmp1);
+                sub(a, m_q_tmp1, d);
             }
         }
     }
@@ -433,8 +387,8 @@ public:
                 del(tmp);
             }
             else {
-                mul(b,c,m_addmul_tmp);
-                sub(a, m_addmul_tmp, d);
+                mul(b,c, m_q_tmp1);
+                sub(a, m_q_tmp1, d);
             }
         }
     }
@@ -500,6 +454,8 @@ public:
 
     void machine_div(mpz const & a, mpz const & b, mpz & c) { mpz_manager<SYNCH>::machine_div(a, b, c); }
 
+    void machine_div_rem(mpz const & a, mpz const & b, mpz & c, mpz & d) { mpz_manager<SYNCH>::machine_div_rem(a, b, c, d); }
+
     void div(mpz const & a, mpz const & b, mpz & c) { mpz_manager<SYNCH>::div(a, b, c); }
     
     void rat_div(mpz const & a, mpz const & b, mpq & c) {
@@ -514,6 +470,13 @@ public:
         reset_denominator(c);
     }
 
+    void machine_idiv_rem(mpq const & a, mpq const & b, mpq & c, mpq & d) {
+        SASSERT(is_int(a) && is_int(b));
+        machine_div_rem(a.m_num, b.m_num, c.m_num, d.m_num);
+        reset_denominator(c);
+        reset_denominator(d);
+    }
+
     void machine_idiv(mpq const & a, mpq const & b, mpz & c) {
         SASSERT(is_int(a) && is_int(b));
         machine_div(a.m_num, b.m_num, c);
@@ -685,7 +648,7 @@ public:
         normalize(a);
     }
 
-    void set(mpq & a, int64 n, uint64 d) {
+    void set(mpq & a, int64_t n, uint64_t d) {
         SASSERT(d != 0);
         set(a.m_num, n);
         set(a.m_den, d);
@@ -717,16 +680,16 @@ public:
 
     void set(mpq & a, char const * val);
 
-    void set(mpz & a, int64 val) { mpz_manager<SYNCH>::set(a, val); }
+    void set(mpz & a, int64_t val) { mpz_manager<SYNCH>::set(a, val); }
 
-    void set(mpq & a, int64 val) {
+    void set(mpq & a, int64_t val) {
         set(a.m_num, val);
         reset_denominator(a);
     }
 
-    void set(mpz & a, uint64 val) { mpz_manager<SYNCH>::set(a, val); }
+    void set(mpz & a, uint64_t val) { mpz_manager<SYNCH>::set(a, val); }
     
-    void set(mpq & a, uint64 val) { 
+    void set(mpq & a, uint64_t val) {
         set(a.m_num, val);
         reset_denominator(a);
     }
@@ -736,13 +699,27 @@ public:
         reset_denominator(a);
     }
 
-    void set(mpz & a, unsigned sz, digit_t const * digits) { mpz_manager<SYNCH>::set(a, sz, digits); }
+    void set(mpz & a, unsigned sz, digit_t const * digits) { 
+        mpz_manager<SYNCH>::set_digits(a, sz, digits); 
+    }
 
     void set(mpq & a, unsigned sz, digit_t const * digits) { 
-        mpz_manager<SYNCH>::set(a.m_num, sz, digits); 
+        mpz_manager<SYNCH>::set_digits(a.m_num, sz, digits); 
         reset_denominator(a); 
     }
 
+    mpq dup(const mpq & source) {
+        mpq temp;
+        set(temp, source);
+        return temp;
+    }
+
+    mpz dup(const mpz & source) {
+        mpz temp;
+        set(temp, source);
+        return temp;
+    }
+
     void swap(mpz & a, mpz & b) { mpz_manager<SYNCH>::swap(a, b); }
 
     void swap(mpq & a, mpq & b) {
@@ -758,17 +735,17 @@ public:
 
     bool is_int64(mpz const & a) const { return mpz_manager<SYNCH>::is_int64(a); }
 
-    uint64 get_uint64(mpz const & a) const { return mpz_manager<SYNCH>::get_uint64(a); }
+    uint64_t get_uint64(mpz const & a) const { return mpz_manager<SYNCH>::get_uint64(a); }
 
-    int64 get_int64(mpz const & a) const { return mpz_manager<SYNCH>::get_int64(a); }
+    int64_t get_int64(mpz const & a) const { return mpz_manager<SYNCH>::get_int64(a); }
 
     bool is_uint64(mpq const & a) const { return is_int(a) && is_uint64(a.m_num); }
 
     bool is_int64(mpq const & a) const { return is_int(a) && is_int64(a.m_num); }
 
-    uint64 get_uint64(mpq const & a) const { SASSERT(is_uint64(a)); return get_uint64(a.m_num); }
+    uint64_t get_uint64(mpq const & a) const { SASSERT(is_uint64(a)); return get_uint64(a.m_num); }
 
-    int64 get_int64(mpq const & a) const { SASSERT(is_int64(a)); return get_int64(a.m_num); }
+    int64_t get_int64(mpq const & a) const { SASSERT(is_int64(a)); return get_int64(a.m_num); }
 
     double get_double(mpz const & a) const { return mpz_manager<SYNCH>::get_double(a); }
 
@@ -784,6 +761,8 @@ public:
 
     unsigned bitsize(mpz const & a) { return mpz_manager<SYNCH>::bitsize(a); }
     unsigned bitsize(mpq const & a) { return is_int(a) ? bitsize(a.m_num) : bitsize(a.m_num) + bitsize(a.m_den); }
+    unsigned storage_size(mpz const & a) { return mpz_manager<SYNCH>::size_info(a); }
+    unsigned storage_size(mpq const & a) { return mpz_manager<SYNCH>::size_info(a.m_num) + mpz_manager<SYNCH>::size_info(a.m_den); }
 
     /**
        \brief Return true if the number is a perfect square, and 
@@ -833,12 +812,15 @@ public:
     }
 
     bool is_even(mpz const & a) { return mpz_manager<SYNCH>::is_even(a); }
-
+public:
     bool is_even(mpq const & a) { return is_int(a) && is_even(a.m_num); }
-
 };
 
+#ifndef SINGLE_THREAD
 typedef mpq_manager<true> synch_mpq_manager;
+#else
+typedef mpq_manager<false> synch_mpq_manager;
+#endif
 typedef mpq_manager<false> unsynch_mpq_manager;
 
 typedef _scoped_numeral<unsynch_mpq_manager> scoped_mpq;
